RUN = "run"
ACTOR = "actor"
PROJECT = "project"
REPO = "repo"
USER = "user"
BOOKMARK = "bookmark"
ARCHIVE = "archive"
PROJECT_SEARCH = "project_search"
ORGANIZATION = "organization"
ORGANIZATION_SEARCH = "organization_search"
PROJECT_DASHBOARD = "project_dashboard"
ORGANIZATION_DASHBOARD = "organization_dashboard"
TAG = "tag"
WORKER = "worker"
AGENT = "agent"
COMPONENT_VERSION = "component_version"
MODEL_VERSION = "model_version"
ARTIFACT_VERSION = "artifact_version"
PRESET = "preset"
QUEUE = "queue"
CONNECTION = "connection"
MEMBER = "member"
INVITATION = "invitation"
TEAM = "team"
SERVICE_ACCOUNT = "service_account"
SERVICE_ACCOUNT_TOKEN = "service_account_token"
POLICY = "policy"
